@@ -1,3 +1,25 @@
+/* GDC -- D front-end for GCC
+   Copyright (C) 2004 David Friedman
+
+   Modified by
+    Michael Parrott, (C) 2009
+    Iain Buclaw, (C) 2010
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+ 
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+ 
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #include "d-gcc-includes.h"
 #include "total.h"
 #include "statement.h"
@@ -181,7 +203,7 @@ d_expand_priv_asm_label(IRState * irs, unsigned n)
 #else
     tree t = d_build_asm_stmt(insnt, NULL_TREE, NULL_TREE, NULL_TREE);
     ASM_VOLATILE_P( t ) = 1;
-    ASM_INPUT_P( t) = 1; // what is this doing?
+    ASM_INPUT_P( t ) = 1; // what is this doing?
     irs->addExp(t);
 #endif
 }
@@ -432,7 +454,7 @@ AsmStatement::toIR(IRState * irs)
 		clobbers_mem = true;
 	    break;
 	case Arg_LocalSize:
-	    var_frame_offset = cfun->x_frame_offset;
+	    var_frame_offset = frame_offset;
 	    if (var_frame_offset < 0)
 		var_frame_offset = - var_frame_offset;
 	    arg_val = irs->integerConstant( var_frame_offset );
