@@ -1,3 +1,24 @@
+/* GDC -- D front-end for GCC
+   Copyright (C) 2004 David Friedman
+
+   Modified by
+    Michael Parrott, (C) 2009
+    Vincenzo Ampolo, Iain Buclaw, (C) 2010
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+ 
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+ 
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
 
 typedef enum {
     Reg_Invalid = -1,
@@ -913,6 +934,7 @@ static AsmOpEnt opData[] = {
     { "pand",     Op_DstSrcMMX },
     { "pandn",    Op_DstSrcMMX },
     { "pavgb",    Op_DstSrcMMX },
+    { "pavgusb",  Op_DstSrcMMX }, // AMD 3dNow! 
     { "pavgw",    Op_DstSrcMMX },
     { "pcmpeqb",  Op_DstSrcMMX },
     { "pcmpeqd",  Op_DstSrcMMX },
@@ -921,6 +943,25 @@ static AsmOpEnt opData[] = {
     { "pcmpgtd",  Op_DstSrcMMX },
     { "pcmpgtw",  Op_DstSrcMMX },
     { "pextrw",   Op_DstSrcImmM }, // gpr32 dest
+    { "pf2id",    Op_DstSrcMMX }, // %% AMD 3dNow! opcodes
+    { "pfacc",    Op_DstSrcMMX },
+    { "pfadd",    Op_DstSrcMMX },
+    { "pfcmpeq",  Op_DstSrcMMX },
+    { "pfcmpge",  Op_DstSrcMMX },
+    { "pfcmpgt",  Op_DstSrcMMX },
+    { "pfmax",    Op_DstSrcMMX },
+    { "pfmin",    Op_DstSrcMMX },
+    { "pfmul",    Op_DstSrcMMX },
+    { "pfnacc",   Op_DstSrcMMX }, // 3dNow values are returned in MM0 register,
+    { "pfpnacc",  Op_DstSrcMMX }, // so should be correct to use Op_DstSrcMMX.
+    { "pfrcp",    Op_DstSrcMMX },
+    { "pfrcpit1", Op_DstSrcMMX },
+    { "pfrcpit2", Op_DstSrcMMX },
+    { "pfrsqit1", Op_DstSrcMMX },
+    { "pfrsqrt",  Op_DstSrcMMX },
+    { "pfsub",    Op_DstSrcMMX },
+    { "pfsubr",   Op_DstSrcMMX },
+    { "pi2fd",    Op_DstSrcMMX }, // %%
     { "pinsrw",   Op_DstSrcImmM }, // gpr32(16), mem16 src, sse too
     { "pmaddwd",  Op_DstSrcMMX },
     { "pmaxsw",   Op_DstSrcMMX },
@@ -928,6 +969,7 @@ static AsmOpEnt opData[] = {
     { "pminsw",   Op_DstSrcMMX },
     { "pminub",   Op_DstSrcMMX },
     { "pmovmskb", Op_DstSrcMMX },
+    { "pmulhrw",  Op_DstSrcMMX }, // AMD 3dNow! 
     { "pmulhuw",  Op_DstSrcMMX },
     { "pmulhw",   Op_DstSrcMMX },
     { "pmullw",   Op_DstSrcMMX },
@@ -965,6 +1007,7 @@ static AsmOpEnt opData[] = {
     { "psubusb",  Op_DstSrcMMX },
     { "psubusw",  Op_DstSrcMMX },
     { "psubw",    Op_DstSrcMMX },
+    { "pswapd",   Op_DstSrcMMX }, // AMD 3dNow! 
     { "punpckhbw", Op_DstSrcMMX },
     { "punpckhdq", Op_DstSrcMMX },
     { "punpckhqdq",Op_DstSrcMMX },
@@ -1403,7 +1446,7 @@ struct AsmProcessor
     }
 
     void addOperand(const char * fmt, AsmArgType type, Expression * e, AsmCode * asmcode, AsmArgMode mode = Mode_Input) {
-	insnTemplate->writestring((char*) fmt);
+	insnTemplate->writestring(fmt);
 	insnTemplate->printf("%d", asmcode->args.dim);
 	asmcode->args.push( new AsmArg(type, e, mode) );
     }
@@ -1483,7 +1526,7 @@ struct AsmProcessor
     }
 
     void writeReg(Reg reg) {
-	insnTemplate->writestring((char*) "%%");
+	insnTemplate->writestring("%%");
 	insnTemplate->write(TREE_STRING_POINTER( regInfo[reg].gccName ),
 	    TREE_STRING_LENGTH( regInfo[reg].gccName ));
     }
@@ -1625,7 +1668,7 @@ struct AsmProcessor
 		if (mnemonic[mlen-1] == 'd')
 		    insnTemplate->write(mnemonic, mlen-1);
 		else {
-		    insnTemplate->writestring((char*) mnemonic);
+		    insnTemplate->writestring(mnemonic);
 		    insnTemplate->writebyte('w');
 		}
 	    }
@@ -1643,7 +1686,7 @@ struct AsmProcessor
 		    insnTemplate->write(mnemonic, mlen-1);
 		    insnTemplate->writebyte('l');
 		} else {
-		    insnTemplate->writestring((char*) mnemonic);
+		    insnTemplate->writestring(mnemonic);
 		}
 	    }
 	    break;
@@ -1675,7 +1718,7 @@ struct AsmProcessor
 	    }
 	    break;
 	default:
-	    insnTemplate->writestring((char*) mnemonic);
+	    insnTemplate->writestring(mnemonic);
 	    if (type_char)
 		insnTemplate->writebyte(type_char);
 	    break;
@@ -1726,11 +1769,11 @@ struct AsmProcessor
 	    stmt->regs |= (1 << Reg_EAX)|
 		(1 << Reg_ECX)|(1 << Reg_EDX);
 
-	insnTemplate->writebyte('\t');
+	insnTemplate->writebyte(' ');
 	for (int i__ = 0; i__ < nOperands; i__++) {
 	    int i;
 	    if (i__ != 0)
-		insnTemplate->writestring((char*) ", ");
+		insnTemplate->writestring(", ");
 
 	    fmt = "%";
     
@@ -1929,7 +1972,7 @@ struct AsmProcessor
 		}
 		if (use_star)
 		    insnTemplate->writebyte('*');
-		if (operand->constDisplacement) {
+		if (operand->constDisplacement || operand->segmentPrefix != Reg_Invalid) {
 		    if (operand->symbolDisplacement.dim)
 			insnTemplate->writebyte('+');
 		    addOperand("%a", Arg_Integer, newIntExp(operand->constDisplacement), asmcode);
@@ -2531,18 +2574,18 @@ struct AsmProcessor
 	// parse primary: DMD allows 'MyAlign' (const int) but not '2+2'
 	// GAS is padding with NOPs last time I checked.
 	Expression * e = parseAsmExp()->optimize(WANTvalue | WANTinterpret);
-	#if V1
+#if V1
 	dinteger_t align = e->toInteger();
-	#else
+#else
 	integer_t align = e->toInteger();
-	#endif
+#endif
 
-	if (align >= 0) {
+	if ((align & -align) == align) {
 	    // %% is this printf portable?
 #ifdef HAVE_GAS_BALIGN_AND_P2ALIGN 
-	    insnTemplate->printf(".balign\t%u", (unsigned) align);
+	    insnTemplate->printf(".balign %u", (unsigned) align);
 #else
-	    insnTemplate->printf(".align\t%u", (unsigned) align);
+	    insnTemplate->printf(".align %u", (unsigned) align);
 #endif
 	} else {
 	    stmt->error("alignment must be a power of 2");
@@ -2554,9 +2597,9 @@ struct AsmProcessor
     void doEven() {
 	// .align for GAS is in bits, others probably use bytes..
 #ifdef HAVE_GAS_BALIGN_AND_P2ALIGN 
-	insnTemplate->writestring((char *) ".align\t2");
+	insnTemplate->writestring(".balign 2");
 #else
-	insnTemplate->writestring((char *) ".align\t2");
+	insnTemplate->writestring(".align 2");
 #endif
 	setAsmCode();
     }
@@ -2572,8 +2615,8 @@ struct AsmProcessor
 
 	machine_mode mode;
 
-	insnTemplate->writestring((char*) directives[op - Op_db]);
-	insnTemplate->writebyte('\t');
+	insnTemplate->writestring(directives[op - Op_db]);
+	insnTemplate->writebyte(' ');
 
 	do {
 	    // DMD is pretty strict here, not even constant expressions are allowed..
@@ -2615,12 +2658,12 @@ struct AsmProcessor
 		    long words[3];
 		    real_to_target(words, & token->float80value.rv(), mode);
 		    // don't use directives..., just use .long like GCC
-		    insnTemplate->printf(".long\t%u", words[0]);
+		    insnTemplate->printf(".long %u", words[0]);
 		    if (mode != SFmode)
 			insnTemplate->printf(",%u", words[1]);
 		    // DMD outputs 10 bytes, so we need to switch to .short here
 		    if (mode == XFmode)
-			insnTemplate->printf("\n\t.short\t%u", words[2]);
+			insnTemplate->printf("\n .short %u", words[2]);
 		} else {
 		    stmt->error("expected float constant");
 		}
