@@ -69,6 +69,7 @@ struct Expression;
 struct DeleteDeclaration;
 struct HdrGenState;
 struct OverloadSet;
+struct AA;
 #if TARGET_NET
 struct PragmaScope;
 #endif
@@ -142,6 +143,7 @@ struct Dsymbol : Object
     virtual int addMember(Scope *sc, ScopeDsymbol *s, int memnum);
     virtual void setScope(Scope *sc);
     virtual void importAll(Scope *sc);
+    virtual void semantic0(Scope *sc);
     virtual void semantic(Scope *sc);
     virtual void semantic2(Scope *sc);
     virtual void semantic3(Scope *sc);
@@ -319,7 +321,11 @@ struct OverloadSet : Dsymbol
 
 struct DsymbolTable : Object
 {
-    StringTable *tab;
+#if STRINGTABLE
+     StringTable *tab;
+#else
+    AA *tab;
+#endif
 
     DsymbolTable();
     ~DsymbolTable();
