@@ -89,6 +89,7 @@ Module::Module(char *filename, Identifier *ident, int doDocComment, int doHdrGen
     decldefs = NULL;
     vmoduleinfo = NULL;
     massert = NULL;
+    munittest = NULL;
     marray = NULL;
     sictor = NULL;
     sctor = NULL;
@@ -132,6 +133,7 @@ Module::Module(char *filename, Identifier *ident, int doDocComment, int doHdrGen
     char *argobj;
     if (global.params.objname)
         argobj = global.params.objname;
+#if 0
     else if (global.params.preservePaths)
         argobj = filename;
     else
@@ -140,6 +142,19 @@ Module::Module(char *filename, Identifier *ident, int doDocComment, int doHdrGen
     {
         argobj = FileName::combine(global.params.objdir, argobj);
     }
+#else // Bugzilla 3547
+    else
+    {
+        if (global.params.preservePaths)
+            argobj = filename;
+        else
+            argobj = FileName::name(filename);
+        if (!FileName::absolute(argobj))
+        {
+            argobj = FileName::combine(global.params.objdir, argobj);
+        }
+    }
+#endif
 
     if (global.params.objname)
         objfilename = new FileName(argobj, 0);
@@ -336,15 +351,20 @@ void Module::read(Loc loc)
 {
     //printf("Module::read('%s') file '%s'\n", toChars(), srcfile->toChars());
     if (srcfile->read())
-    {   error(loc, "cannot read file '%s'", srcfile->toChars());
+    {   error(loc, "is in file '%s' which cannot be read", srcfile->toChars());
         if (!global.gag)
         {   /* Print path
              */
-            for (size_t i = 0; i < global.path->dim; i++)
+            if (global.path)
             {
-                char *p = (char *)global.path->data[i];
-                fprintf(stdmsg, "import path[%d] = %s\n", i, p);
+                for (int i = 0; i < global.path->dim; i++)
+                {
+                    char *p = (char *)global.path->data[i];
+                    fprintf(stdmsg, "import path[%d] = %s\n", i, p);
+                }
             }
+        else
+            fprintf(stdmsg, "Specify path to file '%s' with -I switch\n", srcfile->toChars());
         }
         fatal();
     }
@@ -623,9 +643,16 @@ void Module::parse()
         Dsymbol *prev = dst->lookup(ident);
         assert(prev);
         Module *mprev = prev->isModule();
-        assert(mprev);
-        error(loc, "from file %s conflicts with another module %s from file %s",
-            srcname, mprev->toChars(), mprev->srcfile->toChars());
+        if (mprev)
+            error(loc, "from file %s conflicts with another module %s from file %s",
+                srcname, mprev->toChars(), mprev->srcfile->toChars());
+        else
+        {
+            Package *pkg = prev->isPackage();
+            assert(pkg);
+            error(loc, "from file %s conflicts with package name %s",
+                srcname, pkg->toChars());
+        }
     }
     else
     {
@@ -755,6 +782,14 @@ void Module::semantic()
     }
 #endif
 
+    // Do semantic() on members that don't depend on others
+    for (int i = 0; i < members->dim; i++)
+    {   Dsymbol *s = (Dsymbol *)members->data[i];
+
+        //printf("\tModule('%s'): '%s'.semantic0()\n", toChars(), s->toChars());
+        s->semantic0(sc);
+    }
+
     // Pass 1 semantic routines: do public side of the definition
     for (int i = 0; i < members->dim; i++)
     {   Dsymbol *s = (Dsymbol *)members->data[i];
