@@ -29,6 +29,7 @@
 #if _WIN32
 #include <windows.h>
 #include <direct.h>
+#include <errno.h>
 #endif
 
 #ifndef _WIN32 //#if linux || __APPLE__ || __FreeBSD__
@@ -955,7 +956,13 @@ void FileName::ensurePathExists(const char *path)
 #else
                 if (mkdir(path, 0777))
 #endif
-                    error("cannot create directory %s", path);
+                {
+                    /* Don't error out if another instance of dmd just created
+                     * this directory
+                     */
+                    if (errno != EEXIST)
+                        error("cannot create directory %s", path);
+                }
             }
         }
     }
