@@ -1,6 +1,10 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
    
+   Modified by
+    Michael Parrott, (C) 2009
+    Iain Buclaw, (C) 2010
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -26,13 +30,13 @@ dinteger_t i,
 #else
 integer_t i,
 #endif
-void * p)
+const void * p)
 {
     dt_t * d = new dt_t;
     d->dt = t;
     d->DTnext = 0;
     d->DTint = i;
-    d->DTpointer = p;
+    d->DTpointer = (void*)p;
     return dtcat(pdt, d);
 }
 
